#include "global.h"

#define uart_bit_delay 8

void uart_put_char(char output, FILE *stream);
char uart_get_char(FILE *stream);

void uart_init() {
	static FILE uart_output = FDEV_SETUP_STREAM(uart_put_char, NULL, _FDEV_SETUP_WRITE);
	static FILE uart_input = FDEV_SETUP_STREAM(NULL, uart_get_char, _FDEV_SETUP_READ);
	
	DDRA |= _BV(UTX_PIN);
	DDRA &= ~_BV(URX_PIN);
	
	// Set up stdout and stdin to use the uart
	stdout = &uart_output;
	stdin = &uart_input;
}

void uart_put_char(char output, FILE *stream) {
	cli();
	
	uint8_t lo = PORTA & ~_BV(UTX_PIN);
	uint8_t hi = PORTA | _BV(UTX_PIN);
	
	PORTA = lo;
	_delay_us(uart_bit_delay);
	
	for(uint8_t mask = 1; mask; mask <<= 1) {
		if (output & mask) {
			PORTA = hi;
			} else {
			PORTA = lo;
		}
		_delay_us(uart_bit_delay);
	}
	
	PORTA = hi;
	
	sei();
	
	_delay_us(uart_bit_delay * 2);
}

char uart_get_char(FILE *stream) {
	char output = 0x00;
	
	loop_until_bit_is_clear(PINA, URX_PIN);
	_delay_us(uart_bit_delay * 1.5);
	cli();
	for (uint8_t mask = 1; mask; mask <<= 1) {
		if(bit_is_set(PINA, URX_PIN)) {
			output |= mask;
		}
		_delay_us(uart_bit_delay);
	}
	sei();
	_delay_us(uart_bit_delay/2);
	return output;
}
